Package: biobroom
Title: Turn Bioconductor objects into tidy data frames
Version: 1.43.0
Author: Andrew J. Bass, David G. Robinson, Steve Lianoglou, Emily Nelson, John D. Storey, with
    contributions from Laurent Gatto
Maintainer: John D. Storey <jstorey@princeton.edu> and Andrew J. Bass
 <ajbass@emory.edu>
Description: This package contains methods for converting standard objects
    constructed by bioinformatics packages, especially those in Bioconductor,
    and converting them to tidy data. It thus serves as a complement to the broom
    package, and follows the same the tidy, augment, glance division of tidying
    methods. Tidying data makes it easy to recombine, reshape and visualize
    bioinformatics analyses.
biocViews: MultipleComparison, DifferentialExpression, Regression,
        GeneExpression, Proteomics, DataImport
Depends: R (>= 3.0.0), broom
License: LGPL
LazyData: true
Imports: dplyr, tidyr, Biobase
Suggests: limma, DESeq2, airway, ggplot2, plyr, GenomicRanges,
        testthat, magrittr, edgeR, qvalue, knitr, data.table, MSnbase,
        rmarkdown, SummarizedExperiment
VignetteBuilder: knitr
URL: https://github.com/StoreyLab/biobroom
BugReports: https://github.com/StoreyLab/biobroom/issues
RoxygenNote: 5.0.1
git_url: https://git.bioconductor.org/packages/biobroom
git_branch: devel
git_last_commit: 825b1c8
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 20:14:45 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

S3method(augment,DGEList)
S3method(augment,MArrayLM)
S3method(augment,deSet)
S3method(augment,list)
S3method(augment,qvalue)
S3method(glance,DGEExact)
S3method(glance,GRanges)
S3method(glance,GRangesList)
S3method(glance,MArrayLM)
S3method(glance,deSet)
S3method(glance,list)
S3method(glance,qvalue)
S3method(tidy,DESeqDataSet)
S3method(tidy,DESeqResults)
S3method(tidy,DGEExact)
S3method(tidy,DGEList)
S3method(tidy,EList)
S3method(tidy,ExpressionSet)
S3method(tidy,GRanges)
S3method(tidy,GRangesList)
S3method(tidy,MAList)
S3method(tidy,MArrayLM)
S3method(tidy,MSnSet)
S3method(tidy,RangedSummarizedExperiment)
S3method(tidy,deSet)
S3method(tidy,list)
S3method(tidy,qvalue)
export(augment.DGEList)
export(augment.MArrayLM)
export(augment.deSet)
export(augment.qvalue)
export(augment_sva)
export(glance.DGEExact)
export(glance.GRanges)
export(glance.GRangesList)
export(glance.MArrayLM)
export(glance.deSet)
export(glance.qvalue)
export(glance_sva)
export(tidy.DESeqDataSet)
export(tidy.DESeqResults)
export(tidy.DGEExact)
export(tidy.DGEList)
export(tidy.EList)
export(tidy.ExpressionSet)
export(tidy.GRanges)
export(tidy.GRangesList)
export(tidy.MAList)
export(tidy.MArrayLM)
export(tidy.MSnSet)
export(tidy.RangedSummarizedExperiment)
export(tidy.deSet)
export(tidy.qvalue)
export(tidy_sva)
import(broom)
import(dplyr)
importFrom(Biobase,ExpressionSet)
importFrom(Biobase,assayDataElement)
importFrom(Biobase,assayDataElementNames)
importFrom(Biobase,exprs)
importFrom(tidyr,gather)
importFrom(tidyr,spread)

[![Build Status](https://travis-ci.org/StoreyLab/biobroom.svg?branch=master)](https://travis-ci.org/StoreyLab/biobroom)
<a href="http://www.bioconductor.org/packages/release/bioc/html/edge.html#since"><img border="0" src="http://www.bioconductor.org/shields/years-in-bioc/biobroom.svg"></a> <a href="http://bioconductor.org/packages/stats/bioc/biobroom.html"><img border="0" src="http://www.bioconductor.org/shields/downloads/biobroom.svg"></a> <a href="https://support.bioconductor.org/t/biobroom/"><img border="0" src="http://www.bioconductor.org/shields/posts/biobroom.svg" ></a> <a href="http://www.bioconductor.org/packages/release/bioc/html/biobroom.html#svn_source"><img border="0" src="http://www.bioconductor.org/shields/commits/bioc/biobroom.svg"></a>
biobroom: Tidying up computational biology
====================

This package contains methods for converting standard objects constructed by bioinformatics packages, especially those in [Bioconductor](http://www.bioconductor.org/), and converting them to [tidy data](http://www.jstatsoft.org/v59/i10). It thus serves as a complement to the [broom package](https://github.com/dgrtwo/broom), and follows the same the tidy/augment/glance division of tidying methods. Tidying data makes it easy to recombine, reshape and visualize bioinformatics analyses.

biobroom implements tidying methods for both S3 and S4 classes. Objects that can be tidied include

* ExpressionSet objects
* RangedSummarizedExperiment objects 
* MSnSet objects
* per-gene differential expression tests from limma, edgeR, and DESeq2
* [qvalue](http://www.bioconductor.org/packages/release/bioc/html/qvalue.html) multiple hypothesis testing objects

Installation
------------

The package can be installed with  (requires [devtools](https://github.com/hadley/devtools)):

    devtools::install_github("StoreyLab/biobroom")

Find out more about the provided methods with:

    library(biobroom)
    ?edgeR_tidiers
    ?DESeq2_tidiers
    ?limma_tidiers
    ?ExpressionSet_tidiers
    ?MSnSet_tidiers
    ?GRangesList
    ?GRanges
    ?SummarizedExperiment_tidiers

Note on returned values
-------------

All biobroom `tidy` and `augment` methods, since they tend to be large data frames, return a [tbl_df](http://www.inside-r.org/packages/cran/dplyr/docs/tbl_df) by default (this prevents them from printing many rows at once, while still acting like a traditional data.frame). To change this to a data.frame or data.table, you can set the `biobroom.return` option:

    options(biobroom.return = "data.frame")
    options(biobroom.return = "data.table")

#' Tidying methods for Biobase's SummarizedExperiment objects
#'
#' @param x SummarizedExperiment object
#' @param addPheno whether columns should be included in the tidied output
#' for those in the SummarizedExperiment colData
#' @param assay Which assay to return as the \code{value} column. Defaults to
#'   \code{assays(x)[[1L]]}
#' @param ... extra arguments (not used)
#'
#' @details \code{addPheno=TRUE} adds columns that are redundant (since they
#' add per-sample information to a per-sample-per-gene data frame), but that
#' are useful for some kinds of graphs and analyses.
#'
#' @name SummarizedExperiment_tidiers
#'
#' @examples
#' if (require("SummarizedExperiment", "airway")) {
#'     data(airway)
#'
#'     se <- airway
#'     tidy(se)
#' }
#'
#' @return \code{tidy} returns a data frame with one row per gene-sample
#' combination, with columns
#'   \item{gene}{gene name}
#'   \item{sample}{sample name (from column names)}
#'   \item{value}{expressions}
#'
#' If \code{addPheno} is TRUE then information from colData
#' is added.
#'
#' @S3method tidy RangedSummarizedExperiment
#' @export tidy.RangedSummarizedExperiment
tidy.RangedSummarizedExperiment <- function(x, addPheno=FALSE,
                                            assay=SummarizedExperiment::assayNames(x)[1L],
                                            ...) {
    if (!assay %in% SummarizedExperiment::assayNames(x)) {
        stop("Invalid assay specified: ", assay)
    }
    expressions <- SummarizedExperiment::assays(x)[[assay]] %>%
        fix_data_frame(newcol="gene")
    ret <- expressions %>%
        tidyr::gather(sample, value, -gene) %>%
        dplyr::mutate(sample=as.character(sample))

    if (addPheno) {
        pdat <- as.data.frame(colData(x))
        rownames(pdat) <- colnames(x)
        ret <- cbind(
            ret[, c('gene', 'sample')],
            pdat[ret$sample,,drop=FALSE],
            value=ret$value) %>%
            unrowname
    }
    finish(ret)
}

### Maybe add augment to add regions of interest data

#' Tidying methods for Biobase's ExpressionSet objects
#'
#' @param x MSnSet object
#' @param addPheno whether columns should be included in the tidied output
#' for those in the MSnSet's phenoData
#' @param ... extra arguments (not used)
#'
#' @details \code{addPheno=TRUE} adds columns that are redundant (since they
#' add per-sample information to a per-sample-per-gene data frame), but that
#' are useful for some kinds of graphs and analyses.
#'
#' @name MSnSet_tidiers
#'
#' @examples
#' if (require("MSnbase")) {
#'   library(MSnbase)
#'   # import MSnSet object
#'   data(msnset)
#'
#'   # Use tidy to extract genes, sample ids and measured value
#'   tidy(msnset)
#'   # add phenoType data
#'   tidy(msnset, addPheno=TRUE)
#' }
#' @return \code{tidy} returns a data frame with one row per gene-sample
#' combination, with columns
#'   \item{protein}{protein name}
#'   \item{sample}{sample name (from column names)}
#'   \item{value}{protein quantitation data}
#'
#' @S3method tidy MSnSet
#' @export tidy.MSnSet
tidy.MSnSet <- function(x, addPheno=FALSE, ...) {
    expressions <- fix_data_frame(Biobase::exprs(x), newcol="protein")
    ret <- expressions %>% tidyr::gather(sample.id, value, -protein)

    if (addPheno) {
        pdat <- pData(x)
        ret <- unrowname(as.data.frame(cbind(protein=ret$protein,
                                             sample = ret$sample.id,
                                             pdat[as.character(ret$sample.id), , drop=FALSE],
                                             value=ret$value)))
    }
    finish(ret)
}

#' Tidying methods for a sva list
#'
#' These are methods for turning a sva list, from the sva package, into a tidy data frame.
#' \code{tidy} returns a data.frame of the estimated surrogate variables, \code{glance} returns a data.frame
#' of the posterior probabilities, and \code{glance} returns a
#' data.frame with only the number of surrogate variables.
#'
#' @param x sva list
#' @param data Original data
#' @param ... extra arguments (not used)
#'
#' @return All tidying methods return a \code{data.frame} without rownames.
#' The structure depends on the method chosen.
#'
#' @rdname sva_tidiers
#'
#' @return \code{augment} returns one row per gene. It always
#' contains the columns
#'   \item{pprob.gam}{Posterior probability each gene is affected by heterogeneity}
#'   \item{pprob.b}{Posterior probability each gene is affected by model}
#' @export
augment_sva <- function(x, data, ...) {
    df <- data.frame(pprob.b = x$pprob.b, pprob.gam = x$pprob.gam)
    if (!missing(data)) {
        df <- cbind(as.data.frame(data), df)
    }
    df <- df[, !duplicated(colnames(df))]
    finish(df)
}

#' @rdname sva_tidiers
#'
#' @param addVar add additional coefficients to the estimated surrogate variables
#'
#' @return \code{tidy} returns the estimate surrogate variables.
#' @export
tidy_sva <- function(x, addVar = NULL, ...) {
    df <- data.frame(sv = x$sv)
    colnames(df) <- paste0("sv", 1:dim(df)[2])
    if (!is.null(addVar)) {
        if (nrow(addVar) != nrow(df)) stop("number of rows must be the same as
                                            the number of columns in original data")
        df <- cbind(as.data.frame(addVar), df)
    }
    df <- df[, !duplicated(colnames(df))]
    finish(df)
}
#' @rdname sva_tidiers
#'
#' @return \code{glance} returns the estimate surrogate variables.
#' @export
glance_sva <- function(x, ...) {
    df <- data.frame(n.sv = x$n.sv)
    finish(df)
}

#' @title Convert Bioconductor Object into Tidy Data Frames
#' @name biobroom
#' @description This package contains methods for converting standard objects
#' constructed by bioinformatics packages, especially those in Bioconductor, and
#' converting them to tidy data. It thus serves as a complement to the broom
#' package, and follows the same the tidy, augment, glance division of tidying
#' methods. Tidying data makes it easy to recombine, reshape and visualize
#' bioinformatics analyses.
#' @docType package
#' @aliases biobroom biobroom-package
NULL

#'Tidying methods for GRanges and GRangesList objects.
#'
#' @param x GRanges or GRangesList object
#' @param ... Not used.
#'
#'@return All tidying methods return a \code{data.frame} without rownames. \code{tidy} returns one row for each range, which contains
#'\itemize{
#'\item{start} of the range
#'\item{end} of the range
#'\item{width} (or length) of the range
#'\item{names} of the range
#'\item{strand}
#'\item{seqname} Name of the sequence from which the range comes (usually the chromosome)
#'\item{metadata} Any included metadata, (ie, score, GC content)
#'}
#'For \code{GRangesList}, there will also be a column representing which group the ranges comes from.
#'\code{glance} returns a \code{data.frame} with the number of ranges, the number of sequences, and the number of groups (if applicable).
#'
#'@examples
#' if (require("GenomicRanges", "airway")) {
#' data(airway)
#'
#' # GRangesList object
#' air_gr <- rowRanges(airway)
#'
#' tidy(air_gr)
#' glance(air_gr)
#'
#' # GRanges object
#' air_gr <- rowRanges(airway)@@unlistData
#'
#' tidy(air_gr)
#' glance(air_gr)
#'
#'
#' }

#' @rdname GRanges_tidiers
#' @S3method tidy GRanges
#' @export tidy.GRanges
tidy.GRanges <- function(x, ...) {
  x.dt = as.data.frame(x@ranges)

  x.dt$strand = unlist(mapply(x@strand@values, x@strand@lengths, FUN = function(value, length) {
    as.character(rep(value, length))
  }), use.names=FALSE, recursive=FALSE)

  x.dt$seqname = unlist(mapply(x@seqnames@values, x@seqnames@lengths, FUN = function(value, length) {
    as.character(rep(value, length))
  }), use.names=FALSE, recursive=FALSE)

  x.dt = as.data.frame(cbind(x.dt, x@elementMetadata))

  finish(x.dt)
}


#' @rdname GRanges_tidiers
#' @S3method tidy GRangesList
#' @export tidy.GRangesList
tidy.GRangesList <- function(x, ...) {
  x.dt = tidy(x@unlistData)

  part.dt = as.data.frame(x@partitioning)

  x.dt$item = unlist(mapply(as.factor(part.dt$names), as.numeric(part.dt$width), SIMPLIFY = FALSE, FUN = function(value, length) {
    as.character(rep(value, length))
  }), use.names=FALSE, recursive=FALSE)
  finish(x.dt)
}


#' @rdname GRanges_tidiers
#' @S3method glance GRanges
#' @export glance.GRanges
glance.GRanges <- function(x, ...) {
  x.dt = tidy(x)
  ret = data.frame(ranges = length(gr),
                   sequences = length(unique(x.dt$seqname)))
  finish(ret)
}


#' @rdname GRanges_tidiers
#' @S3method glance GRangesList
#' @export glance.GRangesList
glance.GRangesList <- function(x, ...) {
  x.dt = tidy(x)
  ret = data.frame(ranges = length(gr),
                   sequences = length(unique(x.dt$seqname)),
                   lists = length(unique(x.dt$item)))
  finish(ret)
}

#' Tidying methods for DESeq2 DESeqDataSet objects
#'
#' This reshapes a DESeq2 expressionset object into a tidy format. If the
#' dataset contains hypothesis test results (p-values and estimates), this
#' summarizes one row per gene per possible contrast.
#'
#' @param x DESeqDataSet object
#' @param colData whether colData should be included in the tidied output
#' for those in the DESeqDataSet object. If dataset includes hypothesis test
#' results, this is ignored
#' @param intercept whether to include hypothesis test results from the
#' (Intercept) term. If dataset does not include hypothesis testing,
#' this is ignored
#' @param ... extra arguments (not used)
#'
#' @details \code{colDat=TRUE} adds covariates from colData to the data frame.
#'
#' @return If the dataset contains results (p-values and log2 fold changes),
#' the result is a data frame with the columns
#'   \item{term}{The contrast being tested, as given to
#'   \code{results}}
#'   \item{gene}{gene ID}
#'   \item{baseMean}{mean abundance level}
#'   \item{estimate}{estimated log2 fold change}
#'   \item{stderror}{standard error in log2 fold change estimate}
#'   \item{statistic}{test statistic}
#'   \item{p.value}{p-value}
#'   \item{p.adjusted}{adjusted p-value}
#'
#' If the dataset does not contain results (\code{DESeq} has
#' not been run on it), \code{tidy} defaults to tidying the counts in
#' the dataset:
#'   \item{gene}{gene ID}
#'   \item{sample}{sample ID}
#'   \item{count}{number of reads in this gene in this sample}
#'
#' If \code{colData = TRUE}, it also merges this with the columns present
#' in \code{colData(x)}.
#'
#' @name DESeq2_tidiers
#'
#' @examples
#'
#' # From DESeq2 documentation
#'
#' if (require("DESeq2")) {
#'     dds <- makeExampleDESeqDataSet(betaSD = 1)
#'
#'     tidy(dds)
#'     # With design included
#'     tidy(dds, colData=TRUE)
#'
#'     # add a noise confounding effect
#'     colData(dds)$noise <- rnorm(nrow(colData(dds)))
#'     design(dds) <- (~ condition + noise)
#'
#'     # perform differential expression tests
#'     ddsres <- DESeq(dds, test = "Wald")
#'     # now results are per-gene, per-term
#'     tidied <- tidy(ddsres)
#'     tidied
#'
#'     if (require("ggplot2")) {
#'         ggplot(tidied, aes(p.value)) + geom_histogram(binwidth = .05) +
#'             facet_wrap(~ term, scale = "free_y")
#'     }
#' }
#'
#' @S3method tidy DESeqDataSet
#' @export tidy.DESeqDataSet
tidy.DESeqDataSet <- function(x, colData = FALSE, intercept = FALSE, ...) {
    # try to extract the per-gene, per-coefficient information
    resnames <- DESeq2::resultsNames(x)
    if (length(resnames) > 0) {
        ret <- data.frame(term = resnames) %>%
            dplyr::group_by(term) %>%
            dplyr::do(tidy(DESeq2::results(x, name = as.character(.$term))))
        if (!intercept) {
            ret <- ret %>% dplyr::ungroup() %>% dplyr::filter(term != "(Intercept)")
        }
        return(finish(ret))
    }

    # otherwise, tidy the expression data within it
    expressions <- fix_data_frame(counts(x), newcol="gene")
    ret <- expressions %>%
        tidyr::gather(sample, count, -gene) %>%
        dplyr::mutate(sample=as.character(sample))

    if (colData) {
        cdat <- data.frame(SummarizedExperiment::colData(x), stringsAsFactors=FALSE)
        rownames(cdat) <- colnames(x)
        ret <- cbind(
            ret[, c('gene', 'sample')],
            cdat[ret$sample,,drop=FALSE],
            count=ret$count) %>%
            unrowname
    }
    finish(ret)
}


#' @rdname DESeq2_tidiers
#'
#' @S3method tidy DESeqResults
#' @export tidy.DESeqResults
tidy.DESeqResults <- function(x, ...) {
    nn <- c("baseMean", "estimate", "stderror", "statistic", "p.value", "p.adjusted")
    finish(fix_data_frame(x, newnames = nn, newcol = "gene"))
}

#' @title ExpressionSet results from Hammer et al 2010
#'
#' @docType data
#'
#' @description An ExpressionSet containing the results of the Hammer et al
#' 2010 RNA-Seq study on the nervous system of rats (Hammer et al 2010).
#'
#' This was downloaded from the ReCount database of analysis-ready RNA-Seq
#' datasets (Frazee et al 2011).
#'
#' Hammer, P., Banck, M. S., Amberg, R., Wang, C., Petznick, G., Luo, S.,
#' Khrebtukova, I., Schroth, G. P., Beyerlein, P., and Beutler, A. S. (2010).
#' mRNA-seq with agnostic splice site discovery for nervous system
#' transcriptomics tested in chronic pain. Genome research, 20(6), 847-860.
#' http://www.ncbi.nlm.nih.gov/pmc/articles/PMC2877581/
#'
#' Frazee, A. C., Langmead, B., and Leek, J. T. (2011). ReCount: a
#' multi-experiment resource of analysis-ready RNA-seq gene count datasets.
#' BMC Bioinformatics, 12, 449. http://bowtie-bio.sourceforge.net/recount/
#'
#' @importFrom Biobase ExpressionSet exprs
#' @return ExpressionSet
"hammer"

#' Tidying methods for edge's deSet object
#'
#' @param x deSet object
#' @param data Original data can be added. Default is NULL.
#' @param addPheno whether columns should be included in the tidied output
#' for those in the ExpressionSet's phenoData
#' @param ... extra arguments (not used)
#'
#' @details \code{addPheno=TRUE} adds columns that are redundant (since they
#' add per-sample information to a per-sample-per-gene data frame), but that
#' are useful for some kinds of graphs and analyses.
#'
#' @rdname edge_tidiers
#'
#'
#' @return \code{tidy} returns a data frame with one row per gene-sample
#' combination, with columns
#'   \item{gene}{gene name}
#'   \item{sample}{sample name (from column names)}
#'   \item{value}{expressions on log2 scale}
#'
#' @S3method tidy deSet
#' @export tidy.deSet
tidy.deSet <- function(x, addPheno=FALSE, ...) {
  if (is.null(rownames(exprs(x))) | all(rownames(exprs(x)) == 1:nrow(exprs(x)))) {
    rownames(exprs(x)) <- paste0("g", 1:nrow(exprs(x)))
  }
  if (is.null(colnames(exprs(x))) | all(colnames(exprs(x)) == 1:ncol(exprs(x)))) {
    colnames(exprs(x)) <- paste0("sample", 1:ncol(exprs(x)))
  }
  expressions <- fix_data_frame(Biobase::exprs(x), newcol="gene")
  ret <- expressions %>%
      tidyr::gather(sample, value, -gene)%>%
      dplyr::mutate(sample=as.character(sample))


  if (addPheno) {
    pdat <- pData(x)
    rownames(pdat) <- colnames(exprs(x))
    ret <- unrowname(as.data.frame(cbind(gene=ret$gene,
                                         sample=ret$sample,
                                         pdat[ret$sample, , drop=FALSE],
                                         value=ret$value)))
  }
  finish(ret)
}



#' @rdname edge_tidiers
#'
#' @return \code{augment} returns a data.frame with
#'     \item{p.value}{the original p-values given to \code{qvalue}}
#'     \item{q.value}{the computed q-values}
#'     \item{lfdr}{the local false discovery rate}
#' @S3method augment deSet
#' @export augment.deSet
augment.deSet <- function(x, data, ...) {
    x <- x@qvalueObj
    if (is.null(x)) stop("qvalueObj slot is empty")
    augment(x)
}


#' @rdname edge_tidiers
#'
#' @return \code{glance} returns a data.frame with the model fits
#' @S3method glance deSet
#' @export glance.deSet
glance.deSet <- function(x, ...) {
    df <- data.frame(full.model = as.character(x@full.model)[2],
                     null.model = as.character(x@null.model)[2],
                     stringsAsFactors=FALSE)
    finish(df)
}

#' Tidiers for return values from functions that aren't S3 objects
#'
#' This method handles the return values of functions that return lists
#' rather than S3 objects, such as \code{sva},
#' and therefore cannot be handled by S3 dispatch.
#'
#' @param x list object
#' @param ... extra arguments, passed to the tidying function
#'
#' @details Those tiders themselves are implemented as functions of the
#' form tidy_<function> that are not exported.
#'
#' @name list_tidiers
#'
#' @export
tidy.list <- function(x, ...) {
    if (all( c("sv", "pprob.gam", "pprob.b", "n.sv")
            %in% names(x))) {
        # returned from sva
        tidy_sva(x, ...)
    } else {
        stop("No tidying method recognized for this list")
    }
}


#' @export
augment.list <- function(x, ...) {
    if (all( c("sv", "pprob.gam", "pprob.b", "n.sv")
             %in% names(x))) {
        # returned from sva
        augment_sva(x, ...)
    } else {
        stop("No augmenting method recognized for this list")
    }
}

#' @rdname list_tidiers
#'
#' @export
glance.list <- function(x, ...) {
    if (all( c("sv", "pprob.gam", "pprob.b", "n.sv")
             %in% names(x))) {
        # returned from sva
        glance_sva(x, ...)
    } else {
        stop("No glance method recognized for this list")
    }
}

#' Tidiers for the output of limma (linear models for microarray analysis)
#'
#' Tidy, augment, and glance methods for MArrayLM objects, which contain the
#' results of gene-wise linear models to microarray datasets. This class is
#' the output of the lmFit and eBayes functions.
#'
#' Tidying this fit computes one row per coefficient per gene, while
#' augmenting returns one row per gene, with per-gene statistics included.
#' (This is thus a rare case where the \code{augment} output has more rows
#' than the \code{tidy} output. This is a side effect of the fact that the
#' input to limma is not tidy but rather a one-row-per-gene matrix).
#'
#' @name limma_tidiers
#'
#' @param x \code{MArrayLM}, \code{MAList}, \code{Elist} object
#'
#' @return The output of tidying functions is always a data frame without
#' rownames.
#'
#' @examples
#' if (require("limma")) {
#'     # create random data and design
#'     set.seed(2014)
#'     dat <- matrix(rnorm(1000), ncol=4)
#'     dat[, 1:2] <- dat[, 1:2] + .5  # add an effect
#'     rownames(dat) <- paste0("g", 1:nrow(dat))
#'     des <- data.frame(treatment = c("a", "a", "b", "b"),
#'                       confounding = rnorm(4))
#'
#'     lfit <- lmFit(dat, model.matrix(~ treatment + confounding, des))
#'     eb <- eBayes(lfit)
#'     head(tidy(lfit))
#'     head(tidy(eb))
#'
#'     if (require("ggplot2")) {
#'         # the tidied form puts it in an ideal form for plotting
#'         ggplot(tidy(lfit), aes(estimate)) + geom_histogram(binwidth=1) +
#'             facet_wrap(~ term)
#'         ggplot(tidy(eb), aes(p.value)) + geom_histogram(binwidth=.2) +
#'             facet_wrap(~ term)
#'     }
#' }
NULL


#' @rdname limma_tidiers
#'
#' @param intercept whether the \code{(Intercept)} term should be included
#' (default FALSE)
#'
#' @return \code{tidy} returns one row per gene per coefficient. It always
#' contains the columns
#'   \item{gene}{The name of the gene (extracted from the rownames of the
#'   input matrix)}
#'   \item{term}{The coefficient being estimated}
#'   \item{estimate}{The estimate of each per-gene coefficient}
#'
#' Depending on whether the object comes from \code{eBayes}, it may also
#' contain
#'   \item{statistic}{Empirical Bayes t-statistic}
#'   \item{p.value}{p-value computed from t-statistic}
#'   \item{lod}{log-of-odds score}
#'
#' @import broom
#' @S3method tidy MArrayLM
#' @export tidy.MArrayLM
tidy.MArrayLM <- function(x, intercept = FALSE, ...) {
    coefs <- fix_data_frame(x$coefficients, newnames = colnames(x),
                            newcol="gene")
    ret <- coefs %>% tidyr::gather(term, estimate, -gene)

    # other columns that can be combined (include only those that exist)
    othernames <- c("t", "p.value", "lods")
    newnames <- c("statistic", "p.value", "lod")
    for (i in seq_along(othernames)) {
        if (!is.null(x[[othernames[i]]]))
        ret[[newnames[i]]] <- as.numeric(x[[othernames[i]]])
    }
    if (!intercept) {
        ret <- ret %>% dplyr::filter(term != "(Intercept)") %>%
            dplyr::mutate(term = droplevels(as.factor(term)))
    }

    finish(ret)
}


#' @rdname limma_tidiers
#'
#' @param data original expression matrix; if missing, \code{augment} returns only
#' the computed per-gene statistics
#' @param ... extra arguments, not used
#'
#' @return \code{augment} returns one row per gene, containing the original
#' gene expression matrix if provided. It then adds columns containing
#' the per-gene statistics included in the MArrayLM object, each prepended
#' with a .:
#'   \item{.gene}{gene ID, obtained from the rownames of the input}
#'   \item{.sigma}{per-gene residual standard deviation}
#'   \item{.df.residual}{per-gene residual degrees of freedom}
#'
#' The following columns may also be included, depending on which have been
#' added by \code{lmFit} and \code{eBayes}:
#'   \item{.AMean}{average intensity across probes}
#'   \item{.statistic}{moderated F-statistic}
#'   \item{.p.value}{p-value generated from moderated F-statistic}
#'   \item{.df.total}{total degrees of freedom per gene}
#'   \item{.df.residual}{residual degrees of freedom per gene}
#'   \item{.s2.prior}{prior estimate of residual variance}
#'   \item{.s2.post}{posterior estimate of residual variance}
#'
#' @S3method augment MArrayLM
#' @export augment.MArrayLM
augment.MArrayLM <- function(x, data, ...) {
    cols <- c("sigma", "F", "F.p.value", "AMean", "df.total", "df.residual",
              "s2.prior", "s2.post")
    newnames <- c("sigma", "statistic", "p.value", "AMean", "df.total",
                  "df.residual", "s2.prior", "s2.post")
    lst <- unclass(x)[cols]
    names(lst) <- paste0(".", newnames)
    ret <- cbind(.gene=rownames(x$coefficients), as.data.frame(compact(lst)))

    if (!missing(data)) {
        ret <- cbind(as.data.frame(data), ret)
    }

    finish(ret)
}


#' @rdname limma_tidiers
#'
#' @return \code{glance} returns one row, containing
#'   \item{rank}{rank of design matrix}
#'   \item{df.prior}{empirical Bayesian prior degrees of freedom}
#'   \item{s2.prior}{empirical Bayesian prior residual standard deviation}
#'
#' @S3method glance MArrayLM
#' @export glance.MArrayLM
glance.MArrayLM <- function(x, ...) {
    components <- unclass(x)[c("rank", "df.prior", "s2.prior")]
    as.data.frame(compact(components))
}


#' Tidying method for a MA list
#' @rdname limma_tidiers
#'
#' @return \code{tidy} returns a data frame with one row per gene-sample
#' combination, with columns
#'   \item{gene}{gene name}
#'   \item{sample}{sample name (from column names)}
#'   \item{value}{expressions on log2 scale}
#' @S3method tidy MAList
#' @export tidy.MAList
tidy.MAList <- function(x, ...) {
    tidy_matrix(x$M)
}


#' Tidy an EList expression object
#'
#' @rdname limma_tidiers
#' @param addTargets Add sample level information. Default is FALSE.
#' @return \code{tidy} returns a data frame with one row per gene-sample
#' combination, with columns
#'   \item{gene}{gene name}
#'   \item{sample}{sample name (from column names)}
#'   \item{value}{expressions on log2 scale}
#'   \item{weight}{present if \code{weights} is set}
#'   \item{other columns}{if present and if \code{addTargets} is set}
#' @S3method tidy EList
#' @export tidy.EList
tidy.EList <- function(x, addTargets=FALSE, ...) {
  ret <- tidy_matrix(x$E)
  if (!is.null(x$weights)) {
    rownames(x$weights) <- rownames(x$E)
    ret$weight <- tidy_matrix(x$weights)$value
  }
  if (!is.null(x$sample.weights)) {
      sw <- setNames(x$sample.weights, colnames(x))
      ret$sample.weight <- sw[ret$sample]
  }
  if (addTargets) {
      targets <- x$targets
      rownames(targets) <- colnames(x)
      ret <- cbind(
          ret[, setdiff(names(ret), 'value')],
          targets[ret$sample,,drop=FALSE],
          value=ret$value) %>%
          unrowname
  }
  ret
}

tidy_matrix <- function(x, ...) {
    broom::fix_data_frame(x, newcol = "gene") %>%
        tidyr::gather(sample, value, -gene) %>%
        dplyr::mutate(sample=as.character(sample))
}

unrowname <- function(x) {
    rownames(x) <- NULL
    x
}

compact <- function(x) Filter(Negate(is.null), x)


finish <- function(x) {
    x <- unrowname(x)

    opt <- getOption("biobroom.return", default = "tbl_df")
    if (opt == "tbl_df") {
        dplyr::tbl_df(x)
    } else if (opt == "tbl_dt") {
        dplyr::tbl_dt(x)
    } else if (opt == "data.table") {
        data.table::as.data.table(tbl_dt(x))
    } else if (opt == "data.frame") {
        as.data.frame(x)
    } else {
        stop(paste("Invalid biobroom.return format", opt))
    }
}
# From broom package
fix_data_frame <- function(x, newnames = NULL, newcol = "term", check.names = FALSE)
{
    if (!is.null(newnames) && length(newnames) != ncol(x)) {
        stop("newnames must be NULL or have length equal to number of columns")
    }
    if (all(rownames(x) == seq_len(nrow(x)))) {
        ret <- data.frame(x, stringsAsFactors = FALSE)
        if (!is.null(newnames)) {
            colnames(ret) <- newnames
        }
    }
    else {
        ret <- data.frame(...new.col... = rownames(x), unrowname(x),
                          stringsAsFactors = FALSE, check.names = check.names)
        colnames(ret)[1] <- newcol
        if (!is.null(newnames)) {
            colnames(ret)[-1] <- newnames
        }
    }
    unrowname(ret)
}

## These functions are meant to facilitate testing only

# This data.frame the options for the `type` parameter in
# `makeExampleDataSet` in its `$type` column, as well as other values required
# to create and verify the object that should be created.
.ds.info <- read.table(textConnection("
type                    class          package
DESeq2                  DESeqDataSet   DESeq2
edgeR                   DGEList        edgeR
limma                   EList          edgeR
voom                    EList          edgeR
voomWithQualityWeights  EList          edgeR
ExpressionSet           eSet           Biobase
"), stringsAsFactors=FALSE, header=TRUE)

# Makes example datasets for different objects we tidy.
#
# This is simply a wrapper to the \code{\link[DESeq2]{makeExampleDESeqDataSet}}
# function, which then transforms the output to the appropriate type.

# @param type The type of object to output. So far we can build objects for
#   \itemize{
#    \item{\code{"DESeq2"}}{
#         A \code{\link[DESeq2]{DESeqDataSet}}
#     }
#     \item{\code{"edgeR"}}{
#         A \code{\link[edgeR]{DGEList}}
#     }
#     \item{\code{"limma"}}{
#         An \code{\link[limma]{EList}}
#     }
#     \item{\code{"voom"}}{
#         An \code{\link[limma]{EList}} which results from passing a
#         \code{\link[edgeR]{DGEList}} through \code{\link[limma]{voom}}
#     }
#     \item{\code{"voomWithQualityWeights"}}{
#         Same as voom, but run through
#         \code{\link[limma]{voomWithQualityWeights}}
#     }
#     \item{\code{"ExpressionSet"}}{
#       An \code{\link[Biobase]{ExpressionSet}}
#     }
#   }
#
# @return The desired output object
makeExampleDataSet <-
    function(type = 'DESeq2',
             n = 1000, m = 12, betaSD = 0, interceptMean = 4,
             interceptSD = 2,
             dispMeanRel = function(x) 4/x + 0.1, sizeFactors = rep(1, m)) {
    type <- match.arg(type, .ds.info$type)
    ds.info <- .ds.info[.ds.info$type == type,]
    if (nrow(ds.info) != 1L) {
        stop("Unrecognized data type: ", type ,".\n  Available options are: ",
             paste(.ds.info$type, collapse=',', sep=' '))
    }
    if (!require("DESeq2")) {
        stop("DESeq2 required to create dataset")
    }
    pkg <- ds.info$package
    if (type != 'DESeq2' && !require(pkg, character.only=TRUE)) {
        stop(pkg, " package required to create data of this type")
    }

    dds <- DESeq2::makeExampleDESeqDataSet(n = n, m = m, betaSD = betaSD,
                                           interceptMean = interceptMean,
                                           interceptSD = interceptSD,
                                           dispMeanRel = dispMeanRel)
    dds <- estimateSizeFactors(dds)
    xprs <- log2(counts(dds, normalized=TRUE) + 1)
    pd <- as.data.frame(colData(dds))
    fd <- data.frame(
        gene=rownames(dds),
        seqnames=as.character(seqnames(rowRanges(dds))),
        start=start(rowRanges(dds)),
        end=end(rowRanges(dds)),
        stringsAsFactors=FALSE)

    if (type == 'DESeq2') {
        out <- dds
    } else if (type %in% c('edgeR', 'limma', 'voom', 'voomWithQualityWeights')) {
        out <- DGEList(counts(dds))
        out <- calcNormFactors(out)
        out$samples <- cbind(out$samples, pd)
        out$genes <- fd
        if (type != 'edgeR') {
            mm <- model.matrix(design(dds), out$samples)
            if (type == 'voomWithQualityWeights') {
                out <- voomWithQualityWeights(out, mm, plot=FALSE)
            } else {
                out <- voom(out, mm, plot=FALSE)
            }
            if (type == 'limma') {
                out$weights <- NULL
                out$design <- NULL
                out$targets$group <- NULL
            }
        }
    } else if (type == 'ExpressionSet') {
        out <- ExpressionSet(xprs)
        pData(out) <- pd
        fData(out) <- fd
    } else {
        stop(type, " not yet accounted for")
    }

    if (!is(out, ds.info$class)) {
        warning("The output object is a ", class(out), " but we expected ",
                ds.info$class)
    }

    out
}

context("DESeq2")

test_that("Basic DESeq2 tidier works as expected", {
    dds <- makeExampleDataSet("DESeq2")
    td <- tidy(dds)
    tdp <- tidy(dds, colData=TRUE)

    ## Check that both objects have the bare minimum columns with expected
    ## types
    req.cols <- c(gene='character', sample='character', count='integer')
    for (rc in names(req.cols)) {
        expect_is(td[[rc]], req.cols[[rc]], info=sprintf('td (%s)', rc))
        expect_is(tdp[[rc]], req.cols[[rc]], info=sprintf('tdp (%s)', rc))
    }

    ## tdp has the extra sample data
    pd <- as.data.frame(colData(dds))
    pd.cols <- setNames(sapply(pd, class), names(pd))
    for (pc in names(pd.cols)) {
        expect_is(tdp[[pc]], pd.cols[[pc]], info=sprintf("tdp pheno (%s)", pc))
    }
})

context("limma")

test_that("limma tidier works as expected", {
    dds <- makeExampleDataSet("limma")
    td <- tidy(dds)
    tdp <- tidy(dds, addTargets=TRUE)

    ## Check that both objects have the bare minimum columns with expected
    ## types
    req.cols <- c(gene='character', sample='character', value='numeric')
    for (rc in names(req.cols)) {
        expect_is(td[[rc]], req.cols[[rc]], info=sprintf('td (%s)', rc))
        expect_is(tdp[[rc]], req.cols[[rc]], info=sprintf('tdp (%s)', rc))
    }

    ## tdp has the extra sample data
    pd <- dds$targets
    pd.cols <- setNames(sapply(pd, class), names(pd))
    for (pc in names(pd.cols)) {
        expect_is(tdp[[pc]], pd.cols[[pc]], info=sprintf("tdp pheno (%s)", pc))
    }
})

test_that("voom tidier adds weight column", {
    dds <- makeExampleDataSet("voom")
    td <- tidy(dds)
    tdp <- tidy(dds, addTargets=TRUE)

    ## weights added correctly
    expect_is(td[['weight']], 'numeric')
    expect_is(tdp[['weight']], 'numeric')
    expect_equal(td[['weight']], tdp[['weight']])

    ## vanilla limma tidy has been verified, so ensure that these tidied objects
    ## match base limma tidies objects
    elist <- dds
    elist$weights <- NULL
    elist$design <- NULL

    ld <- tidy(elist)
    ldp <- tidy(elist, addTargets=TRUE)

  #  expect_equal(transform(td, weight=NULL), ld)
   # expect_equal(transform(tdp, weight=NULL), ldp)
})

test_that("voomWithQualityWeights tidier adds weight and sample.weight columns", {
    dds <- makeExampleDataSet("voomWithQualityWeights")
    td <- tidy(dds)
    tdp <- tidy(dds, addTargets=TRUE)

    ## weights and sample.weight added correctly
    expect_is(td[['weight']], 'numeric')
    expect_is(tdp[['weight']], 'numeric')
    expect_equal(td[['weight']], tdp[['weight']])

    expect_is(td[['weight']], 'numeric')
    expect_is(tdp[['weight']], 'numeric')
    expect_equal(td[['weight']], tdp[['weight']])

    ## vanilla limma tidy has been verified, so ensure that these tidied objects
    ## match base limma tidies objects
    elist <- dds
    elist$weights <- NULL
    elist$design <- NULL

    ld <- tidy(elist)
    ldp <- tidy(elist, addTargets=TRUE)

    #expect_equal(transform(td, weight=NULL), ld)
   # expect_equal(transform(tdp, weight=NULL), ldp)
})

context("testing helpers")

test_that("makeExampleDataSet produces dataset of the right type", {
    ds.info <- biobroom:::.ds.info
    for (i in 1:nrow(ds.info)) {
        dtype <- ds.info$type[i]
        eclass <- ds.info$class[i]
        obj <- makeExampleDataSet(dtype)
        expect_is(obj, eclass, info=paste("DataSet for type:", dtype))
    }
})

#' Tidiers for edgeR's differential expression objects
#'
#' Tidy, augment and glance methods for turning edgeR objects into tidy data
#' frames, where each row represents one observation and each column represents
#' one column.
#'
#' @name edgeR_tidiers
#'
#' @param x DGEExact, DGEList object
#' @param data merge data to augment. This is particularly useful when merging
#' gene names or other per-gene information. Default is NULL.
#'
#' @examples
#' if (require("edgeR")) {
#'     library(Biobase)
#'     data(hammer)
#'     hammer.counts <- exprs(hammer)[, 1:4]
#'     hammer.treatment <- phenoData(hammer)$protocol[1:4]
#'
#'     y <- DGEList(counts=hammer.counts,group=hammer.treatment)
#'     y <- calcNormFactors(y)
#'     y <- estimateCommonDisp(y)
#'     y <- estimateTagwiseDisp(y)
#'     et <- exactTest(y)
#'
#'     head(tidy(et))
#'     head(glance(et))
#' }
#'
#' @S3method tidy DGEExact
#' @export tidy.DGEExact
tidy.DGEExact <- function(x, ...) {
    ret <- fix_data_frame(x$table, c("estimate", "logCPM", "p.value"),
                          newcol = "gene")
    finish(ret)
}

#' @rdname edgeR_tidiers
#'
#' @param addSamples Merge information from samples. Default is FALSE.
#'
#' @return \code{tidy} defaults to tidying the counts in
#' the dataset:
#'   \item{gene}{gene ID}
#'   \item{sample}{sample ID}
#'   \item{count}{number of reads in this gene in this sample}
#'
#' If \code{addSamples = TRUE}, it also merges this with the sample information present
#' in \code{x$samples}.
#'
#'
#' @S3method tidy DGEList
#' @export tidy.DGEList
tidy.DGEList <- function(x, addSamples = FALSE, ...) {
    if (is.null(rownames(x$counts)) | all(rownames(x$counts) == 1:nrow(x$counts))) {
      rownames(x$counts) <- paste0("g", 1:nrow(x$counts))
    }
    expressions <- fix_data_frame(x$counts, newnames = colnames(x$counts), newcol="gene")
    ret <- expressions %>%
        tidyr::gather(sample, count, -gene) %>%
        dplyr::mutate(sample=as.character(sample))
    if (addSamples) {
        sdat <- x$samples
        rownames(sdat) <- colnames(x)
        ret <- cbind(
            ret[, c('gene', 'sample')],
            sdat[ret$sample,,drop=FALSE],
            count=ret$count) %>%
            unrowname
    }

    finish(ret)
}

#' @rdname edgeR_tidiers
#' @return \code{augment} returns per-gene information (DGEList only)
#' @S3method augment DGEList
#' @export augment.DGEList
augment.DGEList <- function(x, data = NULL, ...) {
    ret <- list()
    # other columns that can be combined (include only those that exist)
    othernames <- c("genes", "AveLogCPM", "common.dispersion", "trended.dispersion", "tagwise.dispersion")
    #newnames <- c("statistic", "p.value", "lod")
    for (i in seq_along(othernames)) {
        if (!is.null(x[[othernames[i]]]))
            ret[[othernames[i]]] <- as.numeric(x[[othernames[i]]])
    }
    if (!missing(data)) {
        ret <- cbind(as.data.frame(data), as.data.frame(ret))
    }
    if (is.null(names(ret))) stop("No columns to augment in DGEList")
    finish(as.data.frame(ret))
}

#' @rdname edgeR_tidiers
#'
#' @param alpha Confidence level to test for significance
#' @param p.adjust.method Method for adjusting p-values to determine
#' significance; can be any in p.adjust.methods
#' @param ... extra arguments (not used)
#'
#' @return \code{glance} returns one row with the columns (DGEExact only)
#'   \item{significant}{number of significant genes using desired adjustment
#'   method and confidence level}
#'   \item{comparison}{The pair of groups compared by edgeR, delimited by /}
#'
#' @S3method glance DGEExact
#' @export glance.DGEExact
glance.DGEExact <- function(x, alpha=.05, p.adjust.method="fdr", ...) {
    pvals <- x$table$PValue
    pvals.adj <- p.adjust(pvals, p.adjust.method)
    data.frame(significant=sum(pvals.adj <= alpha),
               comparison=paste(x$comparison, collapse="/"))
}

#' Tidying methods for Biobase's ExpressionSet objects
#'
#' @param x ExpressionSet object
#' @param addPheno whether columns should be included in the tidied output
#' for those in the ExpressionSet's phenoData
#' @param assay The name of the \code{\link[Biobase]{assayDataElement}} to use
#'   as the values to tidy. Defaults to \code{assayDataElementNames(x)[1L]},
#'   which is usually equivalent to \code{exprs(x)}.
#' @param ... extra arguments (not used)
#'
#' @details \code{addPheno=TRUE} adds columns that are redundant (since they
#' add per-sample information to a per-sample-per-gene data frame), but that
#' are useful for some kinds of graphs and analyses.
#'
#' @name ExpressionSet_tidiers
#'
#' @examples
#' library(Biobase)
#' # import ExpressionSet object
#' data(hammer)
#'
#' # Use tidy to extract genes, sample ids and measured value
#' tidy(hammer)
#' # add phenoType data
#' tidy(hammer, addPheno=TRUE)
#'
#' @return \code{tidy} returns a data frame with one row per gene-sample
#' combination, with columns
#'   \item{gene}{gene name}
#'   \item{sample}{sample name (from column names)}
#'   \item{value}{expressions on log2 scale}
#'
#' @S3method tidy ExpressionSet
#' @export tidy.ExpressionSet
#' @importFrom Biobase assayDataElement assayDataElementNames pData
tidy.ExpressionSet <- function(x, addPheno=FALSE,
                               assay=Biobase::assayDataElementNames(x)[1L],
                               ...) {
    if (!assay %in% Biobase::assayDataElementNames(x)) {
        stop("Invalid assayDataElementName: ", assay)
    }
    expressions <- Biobase::assayDataElement(x, assay) %>%
        fix_data_frame(newcol="gene")
    ret <- expressions %>%
        tidyr::gather(sample, value, -gene) %>%
        dplyr::mutate(sample=as.character(sample))

    if (addPheno) {
        pdat <- pData(x)
        rownames(pdat) <- colnames(x)
        ret <- cbind(
            ret[, c('gene', 'sample')],
            pdat[ret$sample,,drop=FALSE],
            value=ret$value) %>%
            unrowname
    }
    finish(ret)
}

---
title: "biobroom Vignette"
author: "Andrew J. Bass, Emily Nelson, David Robinson and John D. Storey"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Vignette Title}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---
  
# About biobroom

The `biobroom` package contains methods for converting standard objects in
Bioconductor into a "tidy format". It serves as a complement to the popular [broom](https://github.com/dgrtwo/broom)
package, and follows the same division (`tidy`/`augment`/`glance`) of tidying
methods.

"Tidy data" is a data analysis paradigm that focuses on keeping data formatted as a single observation per row of a data table. For further information, please see [Hadley Wickham's seminal paper](http://vita.had.co.nz/papers/tidy-data.pdf) on the subject.  "Tidy" is not a normative statement about the quality of an object's structure. Rather, it is a technical specification about the choice of rows and columns. A tidied data frame is not "better" than an S4 object; it simply allows analysis with a different set of tools.

Tidying data makes it easy to recombine, reshape and visualize bioinformatics
analyses. Objects that can be tidied include

* `ExpressionSet` object, 
* `GRanges` and `GRangesList` objects,
* `RangedSummarizedExperiment` object,
* `MSnSet` object,
* per-gene differential expression tests from `limma`, `edgeR`, and `DESeq2`,
* `qvalue` object for multiple hypothesis testing.


We are currently working on adding more methods to existing Bioconductor
objects. If any bugs are found please contact the authors or visit our
[github page](https://github.com/StoreyLab/biobroom). Otherwise, any questions
can be answered on the [Bioconductor support site](https://support.bioconductor.org/).

# Installation

```{r global_options, include=FALSE, echo=FALSE}
knitr::opts_chunk$set(fig.width=12, fig.height=6, out.width='700in', out.height='350in', 
                      echo=TRUE, warning=FALSE, message=FALSE, cache=FALSE, dev='png')
```

The `biobroom` package can be installed by typing in a R terminal:

```{r eval=FALSE}
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("biobroom")
```

To find out more about the provided objects:

```{r eval=FALSE}
library(biobroom)
?edgeR_tidiers
?DESeq2_tidiers
?limma_tidiers
?ExpressionSet_tidiers
?MSnSet_tidiers
?qvalue_tidiers
```

# Examples

## qvalue object

The [qvalue package](https://www.bioconductor.org/packages/release/bioc/html/qvalue.html)
is a popular package to estimate q-values and local false discovery rates. To get started,
we can load the `hedenfalk` dataset included in the `qvalue` package:
```{r}
library(qvalue)
data(hedenfalk)

qobj <- qvalue(hedenfalk$p)
names(qobj)
```

`qobj` is a `qvalue` object, generated from the p-values contained in the `hedenfalk` dataset. If we wanted to use a package such as `dplyr` or `ggplot`,
we would need to convert the results into a data frame. The `biobroom` package
makes this conversion easy by using the `tidy`, `augment` and `glance` functions:

- `tidy` returns one row for each choice of the tuning parameter lambda.
- `augment` returns one row for each provided p-value, including the computed q-value and local false discovery rate.
- `glance` returns a single row containing the estimated `pi0`.

Applying these functions to `qobj`:  

```{r}
library(biobroom)
# use tidy/augment/glance to restructure the qvalue object
head(tidy(qobj))
head(augment(qobj))
head(glance(qobj))
```

The original data, or in this example the gene names, can be inputted into `augment` using the `data` argument:
```{r}
# create sample names
df <- data.frame(gene = 1:length(hedenfalk$p))
head(augment(qobj, data = df))
```
The tidied data can be used to easily create plots:

```{r}
library(ggplot2)
# use augmented data to compare p-values to q-values
ggplot(augment(qobj), aes(p.value, q.value)) + geom_point() +
  ggtitle("Simulated P-values versus Computed Q-values") + theme_bw()
```

Additionally, we can extract out important information such as significant
genes under a false discovery rate threshold:
```{r}
library(dplyr)

# Find significant genes under 0.05 threshold
sig.genes <- augment(qobj) %>% filter(q.value < 0.05)
head(sig.genes)
```

## DESeq2 objects

To demonstrate tidying on `DESeq2` objects we have used the published `airway`
RNA-Seq experiment, available as a package from *Bioconductor*:

```{r eval=FALSE}
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("airway")
```

Import the `airway` dataset:

```{r}
library(DESeq2)
library(airway)

data(airway)
airway_se <- airway
```

`airway_se` is a `SummarizedExperiment` object, which is a type of object used by the `DESeq2` package. Next, we create a `DESeqDataSet` object
and show the output of tidying this object:

```{r}
airway_dds <- DESeqDataSet(airway_se, design = ~cell + dex)

head(tidy(airway_dds))
```

Only the gene counts are outputted since there has been no analysis performed.
We perform an analysis on the data and then `tidy` the resulting object:
```{r}
# differential expression analysis
deseq <- DESeq(airway_dds)
results <- results(deseq)
# tidy results
tidy_results <- tidy(results)
head(tidy_results)
```

As an example to show how easy it is to manipulate the resulting object,
`tidy_results`, we can use `ggplot2` to create a volcano plot of the p-values: 

```{r}
ggplot(tidy_results, aes(x=estimate, y=log(p.value),
                         color=log(baseMean))) + geom_point(alpha=0.5) +
  ggtitle("Volcano Plot For Airway Data via DESeq2") + theme_bw()
```

##edgeR objects

Here we use the  `hammer` dataset included in `biobroom` package. `edgeR` can be used to perform differential
expression analysis as follows:

```{r}
library(edgeR)
data(hammer)

hammer.counts <- Biobase::exprs(hammer)[, 1:4]
hammer.treatment <- Biobase::phenoData(hammer)$protocol[1:4]

y <- DGEList(counts=hammer.counts,group=hammer.treatment)
y <- calcNormFactors(y)
y <- estimateCommonDisp(y)
y <- estimateTagwiseDisp(y)
et <- exactTest(y)
```

The results of the analysis are stored in `et`, which is an `DGEExact` object. We can `tidy` this object using `biobroom`:

```{r}
head(tidy(et))
```

`glance` shows a summary of the experiment: the number of genes found significant (at a specified `alpha`), and which contrasts were compared to get the results.

```{r}
glance(et, alpha = 0.05)
```

Additionally, we can can easily manipulate the resulting object and create
a volcano plot of the p-values using `ggplot2`:
```{r}
ggplot(tidy(et), aes(x=estimate, y=log(p.value), color=logCPM)) +
  geom_point(alpha=0.5) + ggtitle("Volcano Plot for Hammer Data via EdgeR") +
  theme_bw()
```

##limma objects

To demonstrate how `biobroom` works with `limma` objects, we generate some simulated data to test the tidier for `limma` objects. 

```{r}
# create random data and design
dat <- matrix(rnorm(1000), ncol=4)
dat[, 1:2] <- dat[, 1:2] + .5  # add an effect
rownames(dat) <- paste0("g", 1:nrow(dat))
des <- data.frame(treatment = c("a", "a", "b", "b"),
                  confounding = rnorm(4))
```

We then use `lmFit` and `eBayes` (functions included in `limma`) to fit a linear model and use `tidy` to convert the
resulting object into tidy format:

```{r}
lfit <- lmFit(dat, model.matrix(~ treatment + confounding, des))
eb <- eBayes(lfit)

head(tidy(lfit))
head(tidy(eb))
```

Analysis can easily be performed from the tidied data. The package `ggplot2` can
be used to make a volcano plot of the p-values:

```{r}
ggplot(tidy(eb), aes(x=estimate, y=log(p.value), color=statistic)) + 
  geom_point() + ggtitle("Nested Volcano Plots for Simulated Data Processed with limma") +
  theme_bw()
  
```


##ExpressionSet objects

`tidy` can also be run directly on `ExpressionSet` objects, as described in another popular `Bioconductor` package `Biobase.` The
`hammer` dataset we used above (which is included in the `biobroom` package) is an `ExpressionSet` object, so we'll use that to demonstrate.

```{r}
library(Biobase)

head(tidy(hammer))
```

We can add the phenotype information by using the argument `addPheno`:
```{r}
head(tidy(hammer, addPheno = TRUE))
```

Now we can easily visualize the distribution of counts for each protocol by using
`ggplot2`:

```{r}
ggplot(tidy(hammer, addPheno=TRUE), aes(x=protocol, y=log(value))) +
  geom_boxplot() + ggtitle("Boxplot Showing Effect of Protocol on Expression")
```

##MSnSet Objects

`tidy` can also be run directly on `MSnSet` objects from `MSnbase`, which as specialised containers for quantitative proteomics data.

```{r}
library(MSnbase)
data(msnset)

head(tidy(msnset))

head(tidy(msnset, addPheno = TRUE))
```

# Note on returned values

All *biobroom* `tidy` and `augment` methods return a `tbl_df` by default (this
prevents them from printing many rows at once, while still acting like a
traditional `data.frame`). To change this to a `data.frame` or `data.table`,
you can set the `biobroom.return` option:

```{r eval=FALSE}
options(biobroom.return = "data.frame")
options(biobroom.return = "data.table")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sva_tidiers.R
\name{augment_sva}
\alias{augment_sva}
\alias{tidy_sva}
\alias{glance_sva}
\title{Tidying methods for a sva list}
\usage{
augment_sva(x, data, ...)

tidy_sva(x, addVar = NULL, ...)

glance_sva(x, ...)
}
\arguments{
\item{x}{sva list}

\item{data}{Original data}

\item{...}{extra arguments (not used)}

\item{addVar}{add additional coefficients to the estimated surrogate variables}
}
\value{
All tidying methods return a \code{data.frame} without rownames.
The structure depends on the method chosen.

\code{augment} returns one row per gene. It always
contains the columns
  \item{pprob.gam}{Posterior probability each gene is affected by heterogeneity}
  \item{pprob.b}{Posterior probability each gene is affected by model}

\code{tidy} returns the estimate surrogate variables.

\code{glance} returns the estimate surrogate variables.
}
\description{
These are methods for turning a sva list, from the sva package, into a tidy data frame.
\code{tidy} returns a data.frame of the estimated surrogate variables, \code{glance} returns a data.frame
of the posterior probabilities, and \code{glance} returns a
data.frame with only the number of surrogate variables.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edgeR_tidiers.R
\name{edgeR_tidiers}
\alias{edgeR_tidiers}
\alias{tidy.DGEExact}
\alias{tidy.DGEList}
\alias{augment.DGEList}
\alias{glance.DGEExact}
\title{Tidiers for edgeR's differential expression objects}
\usage{
\method{tidy}{DGEExact}(x, ...)

\method{tidy}{DGEList}(x, addSamples = FALSE, ...)

\method{augment}{DGEList}(x, data = NULL, ...)

\method{glance}{DGEExact}(x, alpha = 0.05, p.adjust.method = "fdr",
  ...)
}
\arguments{
\item{x}{DGEExact, DGEList object}

\item{...}{extra arguments (not used)}

\item{addSamples}{Merge information from samples. Default is FALSE.}

\item{data}{merge data to augment. This is particularly useful when merging
gene names or other per-gene information. Default is NULL.}

\item{alpha}{Confidence level to test for significance}

\item{p.adjust.method}{Method for adjusting p-values to determine
significance; can be any in p.adjust.methods}
}
\value{
\code{tidy} defaults to tidying the counts in
the dataset:
  \item{gene}{gene ID}
  \item{sample}{sample ID}
  \item{count}{number of reads in this gene in this sample}

If \code{addSamples = TRUE}, it also merges this with the sample information present
in \code{x$samples}.

\code{augment} returns per-gene information (DGEList only)

\code{glance} returns one row with the columns (DGEExact only)
  \item{significant}{number of significant genes using desired adjustment
  method and confidence level}
  \item{comparison}{The pair of groups compared by edgeR, delimited by /}
}
\description{
Tidy, augment and glance methods for turning edgeR objects into tidy data
frames, where each row represents one observation and each column represents
one column.
}
\examples{
if (require("edgeR")) {
    library(Biobase)
    data(hammer)
    hammer.counts <- exprs(hammer)[, 1:4]
    hammer.treatment <- phenoData(hammer)$protocol[1:4]

    y <- DGEList(counts=hammer.counts,group=hammer.treatment)
    y <- calcNormFactors(y)
    y <- estimateCommonDisp(y)
    y <- estimateTagwiseDisp(y)
    et <- exactTest(y)

    head(tidy(et))
    head(glance(et))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SummarizedExperiment_tidiers.R
\name{SummarizedExperiment_tidiers}
\alias{SummarizedExperiment_tidiers}
\alias{tidy.RangedSummarizedExperiment}
\title{Tidying methods for Biobase's SummarizedExperiment objects}
\usage{
\method{tidy}{RangedSummarizedExperiment}(x, addPheno = FALSE,
  assay = SummarizedExperiment::assayNames(x)[1L], ...)
}
\arguments{
\item{x}{SummarizedExperiment object}

\item{addPheno}{whether columns should be included in the tidied output
for those in the SummarizedExperiment colData}

\item{assay}{Which assay to return as the \code{value} column. Defaults to
\code{assays(x)[[1L]]}}

\item{...}{extra arguments (not used)}
}
\value{
\code{tidy} returns a data frame with one row per gene-sample
combination, with columns
  \item{gene}{gene name}
  \item{sample}{sample name (from column names)}
  \item{value}{expressions}

If \code{addPheno} is TRUE then information from colData
is added.
}
\description{
Tidying methods for Biobase's SummarizedExperiment objects
}
\details{
\code{addPheno=TRUE} adds columns that are redundant (since they
add per-sample information to a per-sample-per-gene data frame), but that
are useful for some kinds of graphs and analyses.
}
\examples{
if (require("SummarizedExperiment", "airway")) {
    data(airway)

    se <- airway
    tidy(se)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qvalue_tidiers.R
\name{qvalue_tidiers}
\alias{qvalue_tidiers}
\alias{tidy.qvalue}
\alias{augment.qvalue}
\alias{glance.qvalue}
\title{Tidying methods for a qvalue object}
\usage{
\method{tidy}{qvalue}(x, ...)

\method{augment}{qvalue}(x, data, ...)

\method{glance}{qvalue}(x, ...)
}
\arguments{
\item{x}{qvalue object}

\item{...}{extra arguments (not used)}

\item{data}{Original data}
}
\value{
All tidying methods return a \code{data.frame} without rownames.
The structure depends on the method chosen.

\code{tidy} returns one row for each choice of the tuning
parameter lambda that was considered (argument \code{lambda} to qvalue),
containing
\item{lambda}{the tuning parameter}
\item{pi0}{corresponding estimate of pi0}
\item{smoothed}{whether the estimate has been spline-smoothed)}

If \code{pi0.method="smooth"}, the pi0 estimates and smoothed values both
appear in the table. If \code{pi0.method="bootstrap"}, \code{smoothed}
is FALSE for all entries.

\code{augment} returns a data.frame with
    \item{p.value}{the original p-values given to \code{qvalue}}
    \item{q.value}{the computed q-values}
    \item{lfdr}{the local false discovery rate}

\code{glance} returns a one-row data.frame containing
    \item{pi0}{the estimated pi0 (proportion of nulls)}
    \item{lambda}{lambda used to compute pi0. Note that if pi0 is 1,
    this may be NA since it can be ambiguous which lambda was used}
}
\description{
These are methods for turning a qvalue object, from the qvalue package for
false discovery rate control, into a tidy data frame. \code{augment}
returns a data.frame of the original p-values combined with the computed
q-values and local false discovery rates, \code{tidy} constructs a table
showing how the estimate of pi0 (the proportion of true nulls) depends
on the choice of the tuning parameter lambda, and \code{glance} returns a
data.frame with only the chosen pi0 value.
}
\examples{

library(ggplot2)
if (require("qvalue")) {
set.seed(2014)

# generate p-values from many one sample t-tests: half of them null
oracle <- rep(c(0, .5), each=1000)
pvals <- sapply(oracle, function(mu) t.test(rnorm(15, mu))$p.value)
qplot(pvals)

q <- qvalue(pvals)

tidy(q)
head(augment(q))
glance(q)

# use augmented data to compare p-values to q-values
ggplot(augment(q), aes(p.value, q.value)) + geom_point()

# use tidy see how pi0 estimate changes with lambda, comparing
# to smoothed version
g <- ggplot(tidy(q), aes(lambda, pi0, color=smoothed)) + geom_line()
g

# show the chosen value
g + geom_hline(yintercept=q$pi0, lty=2)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biobroom.R
\docType{package}
\name{biobroom}
\alias{biobroom}
\alias{biobroom-package}
\title{Convert Bioconductor Object into Tidy Data Frames}
\description{
This package contains methods for converting standard objects
constructed by bioinformatics packages, especially those in Bioconductor, and
converting them to tidy data. It thus serves as a complement to the broom
package, and follows the same the tidy, augment, glance division of tidying
methods. Tidying data makes it easy to recombine, reshape and visualize
bioinformatics analyses.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSnSet_tidiers.R
\name{MSnSet_tidiers}
\alias{MSnSet_tidiers}
\alias{tidy.MSnSet}
\title{Tidying methods for Biobase's ExpressionSet objects}
\usage{
\method{tidy}{MSnSet}(x, addPheno = FALSE, ...)
}
\arguments{
\item{x}{MSnSet object}

\item{addPheno}{whether columns should be included in the tidied output
for those in the MSnSet's phenoData}

\item{...}{extra arguments (not used)}
}
\value{
\code{tidy} returns a data frame with one row per gene-sample
combination, with columns
  \item{protein}{protein name}
  \item{sample}{sample name (from column names)}
  \item{value}{protein quantitation data}
}
\description{
Tidying methods for Biobase's ExpressionSet objects
}
\details{
\code{addPheno=TRUE} adds columns that are redundant (since they
add per-sample information to a per-sample-per-gene data frame), but that
are useful for some kinds of graphs and analyses.
}
\examples{
if (require("MSnbase")) {
  library(MSnbase)
  # import MSnSet object
  data(msnset)

  # Use tidy to extract genes, sample ids and measured value
  tidy(msnset)
  # add phenoType data
  tidy(msnset, addPheno=TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GRanges_tidiers.R
\name{tidy.GRanges}
\alias{tidy.GRanges}
\alias{tidy.GRangesList}
\alias{glance.GRanges}
\alias{glance.GRangesList}
\title{Tidying methods for GRanges and GRangesList objects.}
\usage{
\method{tidy}{GRanges}(x, ...)

\method{tidy}{GRangesList}(x, ...)

\method{glance}{GRanges}(x, ...)

\method{glance}{GRangesList}(x, ...)
}
\arguments{
\item{x}{GRanges or GRangesList object}

\item{...}{Not used.}
}
\value{
All tidying methods return a \code{data.frame} without rownames. \code{tidy} returns one row for each range, which contains
\itemize{
\item{start} of the range
\item{end} of the range
\item{width} (or length) of the range
\item{names} of the range
\item{strand}
\item{seqname} Name of the sequence from which the range comes (usually the chromosome)
\item{metadata} Any included metadata, (ie, score, GC content)
}
For \code{GRangesList}, there will also be a column representing which group the ranges comes from.
\code{glance} returns a \code{data.frame} with the number of ranges, the number of sequences, and the number of groups (if applicable).
}
\description{
Tidying methods for GRanges and GRangesList objects.
}
\examples{
if (require("GenomicRanges", "airway")) {
data(airway)

# GRangesList object
air_gr <- rowRanges(airway)

tidy(air_gr)
glance(air_gr)

# GRanges object
air_gr <- rowRanges(airway)@unlistData

tidy(air_gr)
glance(air_gr)


}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list_tidiers.R
\name{list_tidiers}
\alias{list_tidiers}
\alias{tidy.list}
\alias{glance.list}
\title{Tidiers for return values from functions that aren't S3 objects}
\usage{
\method{tidy}{list}(x, ...)

\method{glance}{list}(x, ...)
}
\arguments{
\item{x}{list object}

\item{...}{extra arguments, passed to the tidying function}
}
\description{
This method handles the return values of functions that return lists
rather than S3 objects, such as \code{sva},
and therefore cannot be handled by S3 dispatch.
}
\details{
Those tiders themselves are implemented as functions of the
form tidy_<function> that are not exported.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ExpressionSet_tidiers.R
\name{ExpressionSet_tidiers}
\alias{ExpressionSet_tidiers}
\alias{tidy.ExpressionSet}
\title{Tidying methods for Biobase's ExpressionSet objects}
\usage{
\method{tidy}{ExpressionSet}(x, addPheno = FALSE,
  assay = Biobase::assayDataElementNames(x)[1L], ...)
}
\arguments{
\item{x}{ExpressionSet object}

\item{addPheno}{whether columns should be included in the tidied output
for those in the ExpressionSet's phenoData}

\item{assay}{The name of the \code{\link[Biobase]{assayDataElement}} to use
as the values to tidy. Defaults to \code{assayDataElementNames(x)[1L]},
which is usually equivalent to \code{exprs(x)}.}

\item{...}{extra arguments (not used)}
}
\value{
\code{tidy} returns a data frame with one row per gene-sample
combination, with columns
  \item{gene}{gene name}
  \item{sample}{sample name (from column names)}
  \item{value}{expressions on log2 scale}
}
\description{
Tidying methods for Biobase's ExpressionSet objects
}
\details{
\code{addPheno=TRUE} adds columns that are redundant (since they
add per-sample information to a per-sample-per-gene data frame), but that
are useful for some kinds of graphs and analyses.
}
\examples{
library(Biobase)
# import ExpressionSet object
data(hammer)

# Use tidy to extract genes, sample ids and measured value
tidy(hammer)
# add phenoType data
tidy(hammer, addPheno=TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edge_tidiers.R
\name{tidy.deSet}
\alias{tidy.deSet}
\alias{augment.deSet}
\alias{glance.deSet}
\title{Tidying methods for edge's deSet object}
\usage{
\method{tidy}{deSet}(x, addPheno = FALSE, ...)

\method{augment}{deSet}(x, data, ...)

\method{glance}{deSet}(x, ...)
}
\arguments{
\item{x}{deSet object}

\item{addPheno}{whether columns should be included in the tidied output
for those in the ExpressionSet's phenoData}

\item{...}{extra arguments (not used)}

\item{data}{Original data can be added. Default is NULL.}
}
\value{
\code{tidy} returns a data frame with one row per gene-sample
combination, with columns
  \item{gene}{gene name}
  \item{sample}{sample name (from column names)}
  \item{value}{expressions on log2 scale}

\code{augment} returns a data.frame with
    \item{p.value}{the original p-values given to \code{qvalue}}
    \item{q.value}{the computed q-values}
    \item{lfdr}{the local false discovery rate}

\code{glance} returns a data.frame with the model fits
}
\description{
Tidying methods for edge's deSet object
}
\details{
\code{addPheno=TRUE} adds columns that are redundant (since they
add per-sample information to a per-sample-per-gene data frame), but that
are useful for some kinds of graphs and analyses.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DESeq2_tidiers.R
\name{DESeq2_tidiers}
\alias{DESeq2_tidiers}
\alias{tidy.DESeqDataSet}
\alias{tidy.DESeqResults}
\title{Tidying methods for DESeq2 DESeqDataSet objects}
\usage{
\method{tidy}{DESeqDataSet}(x, colData = FALSE, intercept = FALSE, ...)

\method{tidy}{DESeqResults}(x, ...)
}
\arguments{
\item{x}{DESeqDataSet object}

\item{colData}{whether colData should be included in the tidied output
for those in the DESeqDataSet object. If dataset includes hypothesis test
results, this is ignored}

\item{intercept}{whether to include hypothesis test results from the
(Intercept) term. If dataset does not include hypothesis testing,
this is ignored}

\item{...}{extra arguments (not used)}
}
\value{
If the dataset contains results (p-values and log2 fold changes),
the result is a data frame with the columns
  \item{term}{The contrast being tested, as given to
  \code{results}}
  \item{gene}{gene ID}
  \item{baseMean}{mean abundance level}
  \item{estimate}{estimated log2 fold change}
  \item{stderror}{standard error in log2 fold change estimate}
  \item{statistic}{test statistic}
  \item{p.value}{p-value}
  \item{p.adjusted}{adjusted p-value}

If the dataset does not contain results (\code{DESeq} has
not been run on it), \code{tidy} defaults to tidying the counts in
the dataset:
  \item{gene}{gene ID}
  \item{sample}{sample ID}
  \item{count}{number of reads in this gene in this sample}

If \code{colData = TRUE}, it also merges this with the columns present
in \code{colData(x)}.
}
\description{
This reshapes a DESeq2 expressionset object into a tidy format. If the
dataset contains hypothesis test results (p-values and estimates), this
summarizes one row per gene per possible contrast.
}
\details{
\code{colDat=TRUE} adds covariates from colData to the data frame.
}
\examples{

# From DESeq2 documentation

if (require("DESeq2")) {
    dds <- makeExampleDESeqDataSet(betaSD = 1)

    tidy(dds)
    # With design included
    tidy(dds, colData=TRUE)

    # add a noise confounding effect
    colData(dds)$noise <- rnorm(nrow(colData(dds)))
    design(dds) <- (~ condition + noise)

    # perform differential expression tests
    ddsres <- DESeq(dds, test = "Wald")
    # now results are per-gene, per-term
    tidied <- tidy(ddsres)
    tidied

    if (require("ggplot2")) {
        ggplot(tidied, aes(p.value)) + geom_histogram(binwidth = .05) +
            facet_wrap(~ term, scale = "free_y")
    }
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/limma_tidiers.R
\name{limma_tidiers}
\alias{limma_tidiers}
\alias{tidy.MArrayLM}
\alias{augment.MArrayLM}
\alias{glance.MArrayLM}
\alias{tidy.MAList}
\alias{tidy.EList}
\title{Tidiers for the output of limma (linear models for microarray analysis)}
\usage{
\method{tidy}{MArrayLM}(x, intercept = FALSE, ...)

\method{augment}{MArrayLM}(x, data, ...)

\method{glance}{MArrayLM}(x, ...)

\method{tidy}{MAList}(x, ...)

\method{tidy}{EList}(x, addTargets = FALSE, ...)
}
\arguments{
\item{x}{\code{MArrayLM}, \code{MAList}, \code{Elist} object}

\item{intercept}{whether the \code{(Intercept)} term should be included
(default FALSE)}

\item{...}{extra arguments, not used}

\item{data}{original expression matrix; if missing, \code{augment} returns only
the computed per-gene statistics}

\item{addTargets}{Add sample level information. Default is FALSE.}
}
\value{
The output of tidying functions is always a data frame without
rownames.

\code{tidy} returns one row per gene per coefficient. It always
contains the columns
  \item{gene}{The name of the gene (extracted from the rownames of the
  input matrix)}
  \item{term}{The coefficient being estimated}
  \item{estimate}{The estimate of each per-gene coefficient}

Depending on whether the object comes from \code{eBayes}, it may also
contain
  \item{statistic}{Empirical Bayes t-statistic}
  \item{p.value}{p-value computed from t-statistic}
  \item{lod}{log-of-odds score}

\code{augment} returns one row per gene, containing the original
gene expression matrix if provided. It then adds columns containing
the per-gene statistics included in the MArrayLM object, each prepended
with a .:
  \item{.gene}{gene ID, obtained from the rownames of the input}
  \item{.sigma}{per-gene residual standard deviation}
  \item{.df.residual}{per-gene residual degrees of freedom}

The following columns may also be included, depending on which have been
added by \code{lmFit} and \code{eBayes}:
  \item{.AMean}{average intensity across probes}
  \item{.statistic}{moderated F-statistic}
  \item{.p.value}{p-value generated from moderated F-statistic}
  \item{.df.total}{total degrees of freedom per gene}
  \item{.df.residual}{residual degrees of freedom per gene}
  \item{.s2.prior}{prior estimate of residual variance}
  \item{.s2.post}{posterior estimate of residual variance}

\code{glance} returns one row, containing
  \item{rank}{rank of design matrix}
  \item{df.prior}{empirical Bayesian prior degrees of freedom}
  \item{s2.prior}{empirical Bayesian prior residual standard deviation}

\code{tidy} returns a data frame with one row per gene-sample
combination, with columns
  \item{gene}{gene name}
  \item{sample}{sample name (from column names)}
  \item{value}{expressions on log2 scale}

\code{tidy} returns a data frame with one row per gene-sample
combination, with columns
  \item{gene}{gene name}
  \item{sample}{sample name (from column names)}
  \item{value}{expressions on log2 scale}
  \item{weight}{present if \code{weights} is set}
  \item{other columns}{if present and if \code{addTargets} is set}
}
\description{
Tidy, augment, and glance methods for MArrayLM objects, which contain the
results of gene-wise linear models to microarray datasets. This class is
the output of the lmFit and eBayes functions.
}
\details{
Tidying this fit computes one row per coefficient per gene, while
augmenting returns one row per gene, with per-gene statistics included.
(This is thus a rare case where the \code{augment} output has more rows
than the \code{tidy} output. This is a side effect of the fact that the
input to limma is not tidy but rather a one-row-per-gene matrix).
}
\examples{
if (require("limma")) {
    # create random data and design
    set.seed(2014)
    dat <- matrix(rnorm(1000), ncol=4)
    dat[, 1:2] <- dat[, 1:2] + .5  # add an effect
    rownames(dat) <- paste0("g", 1:nrow(dat))
    des <- data.frame(treatment = c("a", "a", "b", "b"),
                      confounding = rnorm(4))

    lfit <- lmFit(dat, model.matrix(~ treatment + confounding, des))
    eb <- eBayes(lfit)
    head(tidy(lfit))
    head(tidy(eb))

    if (require("ggplot2")) {
        # the tidied form puts it in an ideal form for plotting
        ggplot(tidy(lfit), aes(estimate)) + geom_histogram(binwidth=1) +
            facet_wrap(~ term)
        ggplot(tidy(eb), aes(p.value)) + geom_histogram(binwidth=.2) +
            facet_wrap(~ term)
    }
}
}
